// src/bridge/core.ts
import { NOOP } from "@vue/devtools-shared";
import mitt from "mitt";
import { parse } from "@vue/devtools-kit";
var Bridge = class {
  constructor(adapter = {
    tracker: NOOP,
    trigger: NOOP
  }) {
    this.emitter = mitt();
    this.adapter = adapter;
    this.adapter.tracker((message) => {
      this._emit(message.event, message.data);
    });
  }
  _on(eventName, handler) {
    this.emitter.on(eventName, handler);
  }
  off(eventName, handler) {
    this.emitter.off(eventName, handler);
  }
  _emit(eventName, data) {
    this.emitter.emit(eventName, data);
  }
  on(eventName, handler) {
    this._on(eventName, handler);
    return () => this.off(eventName, handler);
  }
  once(eventName, handler) {
    const onceHandler = (...args) => {
      handler(...args);
      this.off(eventName, onceHandler);
    };
    this._on(eventName, onceHandler);
  }
  emit(eventName, data) {
    this.adapter.trigger({
      event: eventName,
      data
    });
    this._emit(eventName, data);
  }
  removeAllListeners() {
    this.emitter.all.clear();
  }
};
var bridgeRpcEvents = {
  toggleComponentInspector: "toggle-component-inspector",
  inspectComponentInspector: "inspect-component-inspector",
  scrollToComponent: "scroll-to-component",
  componentBoundingRect: "component-bounding-rect",
  inspectorTree: "inspector-tree",
  inspectorState: "inspector-state",
  updateInspectorTreeId: "inspector-tree:update-id",
  state: "state",
  timelineLayer: "timeline-layer",
  routerInfo: "router-info",
  router: "router",
  routeMatched: "route-matched",
  editState: "edit-inspector-state",
  openInEditor: "open-in-editor",
  toggleApp: "toggle-app",
  isVueInspectorDetected: "vue-inspector:detected",
  enableVueInspector: "vue-inspector:enable",
  unhighlightElement: "element:unhighlight"
};
var BridgeRpcCore = class {
  constructor(_bridge) {
    this.bridge = _bridge;
  }
  on(eventName, handler) {
    this.bridge.on(`${eventName}:req`, async (payload) => {
      const res = await handler(payload);
      this.bridge.emit(`${eventName}:res`, res);
    });
  }
  emit(eventName, payload) {
    return new Promise((resolve) => {
      this.bridge.once(`${eventName}:res`, (payload2) => {
        const res = {
          data: parse(payload2)
        };
        resolve(res);
      });
      this.bridge.emit(`${eventName}:req`, payload);
    });
  }
};

// src/bridge/devtools.ts
import { parse as parse2 } from "@vue/devtools-kit";

// src/vite-rpc/client.ts
import { createRPCClient } from "vite-dev-rpc";
function setupViteRPCClient(ctx, options = {}) {
  if (!ctx)
    return null;
  const { moduleUpdated = () => {
  }, assetsUpdated = () => {
  } } = options;
  const rpcClient = createRPCClient("vite-plugin-vue-devtools", ctx, {
    moduleUpdated,
    assetsUpdated
  }, {
    timeout: -1
  });
  return rpcClient;
}

// src/vite-rpc/server.ts
import { createRPCServer } from "vite-dev-rpc";
function setupViteRPCServer(ws, functions) {
  const rpcServer = createRPCServer("vite-plugin-vue-devtools", ws, functions, {
    timeout: -1
  });
  return rpcServer;
}

// src/bridge/types.ts
var BridgeEvents = /* @__PURE__ */ ((BridgeEvents2) => {
  BridgeEvents2["DEVTOOLS_STATE_UPDATED"] = "devtools-state:updated";
  BridgeEvents2["CUSTOM_TABS_UPDATED"] = "custom-tabs:updated";
  BridgeEvents2["CUSTOM_COMMANDS_UPDATED"] = "custom-commands:updated";
  BridgeEvents2["ROUTER_INFO_UPDATED"] = "router-info:updated";
  BridgeEvents2["ROUTER"] = "router";
  BridgeEvents2["APP_CONNECTED"] = "user-app:connected";
  BridgeEvents2["CLIENT_READY"] = "devtools:client-ready";
  BridgeEvents2["SEND_INSPECTOR_TREE"] = "inspector-tree:send";
  BridgeEvents2["SEND_INSPECTOR_STATE"] = "inspector-state:send";
  BridgeEvents2["EDIT_STATE"] = "state:edit";
  BridgeEvents2["ADD_TIMELINE_EVENT"] = "timeline:add-event";
  return BridgeEvents2;
})(BridgeEvents || {});

// src/bridge/devtools.ts
var devtoolsBridge = {
  value: null,
  viteRpc: {
    enabled: false,
    api: null,
    on: {
      moduleUpdated() {
      },
      assetsUpdated() {
      }
    },
    off: {
      moduleUpdated() {
      },
      assetsUpdated() {
      }
    }
  },
  rpc: null
};
function registerBridgeRpc(options) {
  devtoolsBridge.value = options.bridge;
  devtoolsBridge.rpc = new BridgeRpcCore(options.bridge);
  const moduleUpdatedFn = [];
  const assetsUpdatedFn = [];
  const rpc = setupViteRPCClient(options.viteRPCContext, {
    moduleUpdated: () => {
      moduleUpdatedFn.forEach((fn) => fn());
    },
    assetsUpdated: () => {
      assetsUpdatedFn.forEach((fn) => fn());
    }
  });
  if (rpc) {
    devtoolsBridge.viteRpc = {
      enabled: true,
      api: rpc,
      on: {
        moduleUpdated(fn) {
          moduleUpdatedFn.push(fn);
        },
        assetsUpdated(fn) {
          assetsUpdatedFn.push(fn);
        }
      },
      off: {
        moduleUpdated() {
          moduleUpdatedFn.length = 0;
        },
        assetsUpdated() {
          assetsUpdatedFn.length = 0;
        }
      }
    };
  }
}
var BridgeRpc = class {
  static updateInspectorTreeId(id) {
    devtoolsBridge.rpc.emit(bridgeRpcEvents.updateInspectorTreeId, id);
  }
  static toggleComponentInspector(payload) {
    devtoolsBridge.rpc.emit(bridgeRpcEvents.toggleComponentInspector, payload);
  }
  static inspectComponentInspector() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.inspectComponentInspector);
  }
  static unhighlightElement() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.unhighlightElement);
  }
  static scrollToComponent(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.scrollToComponent, payload);
  }
  static async getComponentBoundingRect(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.componentBoundingRect, payload);
  }
  static getInspectorTree(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.inspectorTree, payload);
  }
  static async getInspectorState(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.inspectorState, payload);
  }
  static async editInspectorState(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.editState, payload);
  }
  static async getDevToolsState() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.state);
  }
  static async getTimelineLayer() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.timelineLayer);
  }
  static async getRouterInfo() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.routerInfo);
  }
  static async navigate(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.router, JSON.stringify(payload));
  }
  static async getMatchedRoutes(path) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.routeMatched, path);
  }
  static async openInEditor(payload) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.openInEditor, JSON.stringify(payload));
  }
  static async isVueInspectorDetected() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.isVueInspectorDetected);
  }
  static async enableVueInspector() {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.enableVueInspector);
  }
  static async toggleApp(id) {
    return devtoolsBridge.rpc.emit(bridgeRpcEvents.toggleApp, id);
  }
  // assets
  static getStaticAssets() {
    return devtoolsBridge.viteRpc.api.getStaticAssets();
  }
  static getImageMeta(filepath) {
    return devtoolsBridge.viteRpc.api.getImageMeta(filepath);
  }
  static getTextAssetContent(filepath, limit = 300) {
    return devtoolsBridge.viteRpc.api.getTextAssetContent(filepath, limit);
  }
  static root() {
    return devtoolsBridge.viteRpc.api.root();
  }
  // graph
  static getGraph() {
    return devtoolsBridge.viteRpc.api.getGraph();
  }
  // graph module udpated
  static graphModuleUpdated(fn) {
    devtoolsBridge.viteRpc.on.moduleUpdated(fn);
    return () => devtoolsBridge.viteRpc.off.moduleUpdated();
  }
  // assets updated
  static assetsUpdated(fn) {
    devtoolsBridge.viteRpc.on.assetsUpdated(fn);
    return () => devtoolsBridge.viteRpc.off.assetsUpdated();
  }
};
BridgeRpc.on = {
  inspectorTreeUpdated(cb, options) {
    devtoolsBridge.value.on("inspector-tree:send" /* SEND_INSPECTOR_TREE */, (payload) => {
      const _payload = parse2(payload);
      options.inspectorId === _payload.inspectorId && cb(_payload);
    });
  },
  inspectorStateUpdated(cb, options) {
    devtoolsBridge.value.on("inspector-state:send" /* SEND_INSPECTOR_STATE */, (payload) => {
      const _payload = parse2(payload);
      options.inspectorId === _payload.inspectorId && cb({
        state: _payload.state,
        getters: _payload.getters
      });
    });
  },
  devtoolsStateUpdated(cb) {
    devtoolsBridge.value.on("devtools-state:updated" /* DEVTOOLS_STATE_UPDATED */, (payload) => {
      cb(parse2(payload));
    });
  },
  customTabsUpdated(cb) {
    devtoolsBridge.value.on("custom-tabs:updated" /* CUSTOM_TABS_UPDATED */, (payload) => {
      cb(parse2(payload));
    });
  },
  customCommandsUpdated(cb) {
    devtoolsBridge.value.on("custom-commands:updated" /* CUSTOM_COMMANDS_UPDATED */, (payload) => {
      cb(parse2(payload));
    });
  },
  routerInfoUpdated(cb) {
    devtoolsBridge.value.on("router-info:updated" /* ROUTER_INFO_UPDATED */, (payload) => {
      cb(parse2(payload));
    });
  },
  addTimelineEvent(cb) {
    devtoolsBridge.value.on("timeline:add-event" /* ADD_TIMELINE_EVENT */, (payload) => {
      cb(parse2(payload));
    });
  }
};

// src/bridge/user-app.ts
import { devtools, stringify } from "@vue/devtools-kit";
import { isBrowser } from "@vue/devtools-shared";
function registerBridgeRpc2(bridge) {
  const bridgeRpcCore = new BridgeRpcCore(bridge);
  bridgeRpcCore.on(bridgeRpcEvents.state, () => {
    var _a, _b;
    return JSON.stringify({
      connected: devtools.state.connected,
      clientConnected: devtools.state.clientConnected,
      vueVersion: ((_b = (_a = devtools.state) == null ? void 0 : _a.activeAppRecord) == null ? void 0 : _b.version) || "",
      tabs: devtools.state.tabs,
      commands: devtools.state.commands,
      vitePluginDetected: devtools.state.vitePluginDetected,
      appRecords: devtools.state.appRecords.map((item) => ({
        id: item.id,
        name: item.name,
        version: item.version,
        routerId: item.routerId,
        moduleDetectives: item.moduleDetectives
      })),
      activeAppRecordId: devtools.state.activeAppRecordId
    });
  });
  bridgeRpcCore.on(bridgeRpcEvents.timelineLayer, () => {
    return JSON.stringify(devtools.context.timelineLayer);
  });
  bridgeRpcCore.on(bridgeRpcEvents.toggleComponentInspector, (payload) => {
    devtools.api.toggleComponentInspector(payload);
    return Promise.resolve(JSON.stringify({}));
  });
  bridgeRpcCore.on(bridgeRpcEvents.inspectComponentInspector, () => {
    return devtools.api.inspectComponentInspector();
  });
  bridgeRpcCore.on(bridgeRpcEvents.unhighlightElement, () => {
    devtools.api.unhighlightElement();
    return Promise.resolve(JSON.stringify({}));
  });
  bridgeRpcCore.on(bridgeRpcEvents.scrollToComponent, (payload) => {
    devtools.api.scrollToComponent(payload);
    return Promise.resolve(JSON.stringify({}));
  });
  bridgeRpcCore.on(bridgeRpcEvents.componentBoundingRect, async (payload) => {
    return devtools.api.getComponentBoundingRect(payload);
  });
  bridgeRpcCore.on(bridgeRpcEvents.inspectorTree, (payload) => {
    return devtools.api.getInspectorTree(payload);
  });
  bridgeRpcCore.on(bridgeRpcEvents.updateInspectorTreeId, async (payload) => {
    devtools.context.activeInspectorTreeId = payload;
  });
  bridgeRpcCore.on(bridgeRpcEvents.inspectorState, async (payload) => {
    return devtools.api.getInspectorState(payload);
  });
  bridgeRpcCore.on(bridgeRpcEvents.editState, async (payload) => {
    return devtools.api.editInspectorState(payload);
  });
  bridgeRpcCore.on(bridgeRpcEvents.routerInfo, () => {
    return JSON.stringify(devtools.context.routerInfo);
  });
  bridgeRpcCore.on(bridgeRpcEvents.router, (payload) => {
    var _a;
    (_a = devtools.context.router) == null ? void 0 : _a.push(JSON.parse(payload)).catch((e) => e);
    return Promise.resolve(JSON.stringify({}));
  });
  bridgeRpcCore.on(bridgeRpcEvents.openInEditor, async (payload) => {
    return devtools.api.openInEditor(JSON.parse(payload));
  });
  bridgeRpcCore.on(bridgeRpcEvents.isVueInspectorDetected, async () => {
    var _a, _b, _c;
    return !!await ((_c = (_b = (_a = devtools) == null ? void 0 : _a.api) == null ? void 0 : _b.getVueInspector) == null ? void 0 : _c.call(_b));
  });
  bridgeRpcCore.on(bridgeRpcEvents.enableVueInspector, async () => {
    var _a, _b, _c;
    const inspector = await ((_c = (_b = (_a = devtools) == null ? void 0 : _a.api) == null ? void 0 : _b.getVueInspector) == null ? void 0 : _c.call(_b));
    if (inspector)
      await inspector.enable();
  });
  bridgeRpcCore.on(bridgeRpcEvents.routeMatched, (payload) => {
    var _a, _b;
    const c = console.warn;
    console.warn = () => {
    };
    const matched = (_b = (_a = devtools.context.router) == null ? void 0 : _a.resolve({
      path: payload || "/"
    }).matched) != null ? _b : [];
    console.warn = c;
    return JSON.stringify(matched);
  });
  bridgeRpcCore.on(bridgeRpcEvents.toggleApp, async (id) => {
    await devtools.api.toggleApp(id);
    return JSON.stringify({});
  });
  function subscribeDevToolsApiUpdated() {
    var _a, _b, _c;
    bridge.emit("devtools-state:updated" /* DEVTOOLS_STATE_UPDATED */, JSON.stringify({
      vueVersion: ((_b = (_a = devtools.state) == null ? void 0 : _a.activeAppRecord) == null ? void 0 : _b.version) || "",
      connected: true,
      clientConnected: devtools.state.clientConnected,
      activeAppRecordId: (_c = devtools.state) == null ? void 0 : _c.activeAppRecordId,
      appRecords: devtools.state.appRecords.map((item) => ({
        id: item.id,
        name: item.name,
        version: item.version,
        routerId: item.routerId,
        moduleDetectives: item.moduleDetectives
      }))
    }));
    devtools.api.on.devtoolsStateUpdated((payload) => {
      var _a2;
      bridge.emit("devtools-state:updated" /* DEVTOOLS_STATE_UPDATED */, JSON.stringify({
        vueVersion: ((_a2 = payload == null ? void 0 : payload.activeAppRecord) == null ? void 0 : _a2.version) || "",
        connected: payload.connected,
        clientConnected: payload.clientConnected,
        appRecords: payload.appRecords.map((item) => ({
          id: item.id,
          name: item.name,
          version: item.version,
          routerId: item.routerId,
          moduleDetectives: item.moduleDetectives
        })),
        activeAppRecordId: payload.activeAppRecordId
      }));
    });
    devtools.api.on.customTabsUpdated((payload) => {
      bridge.emit("custom-tabs:updated" /* CUSTOM_TABS_UPDATED */, JSON.stringify(payload));
    });
    devtools.api.on.customCommandsUpdated((payload) => {
      bridge.emit("custom-commands:updated" /* CUSTOM_COMMANDS_UPDATED */, JSON.stringify(payload));
    });
    devtools.api.on.routerInfoUpdated((payload) => {
      bridge.emit("router-info:updated" /* ROUTER_INFO_UPDATED */, JSON.stringify(payload));
    });
    devtools.api.on.sendInspectorTree((payload) => {
      bridge.emit("inspector-tree:send" /* SEND_INSPECTOR_TREE */, payload);
    });
    devtools.api.on.sendInspectorState((payload) => {
      bridge.emit("inspector-state:send" /* SEND_INSPECTOR_STATE */, payload);
    });
    devtools.api.on.addTimelineEvent((payload) => {
      bridge.emit("timeline:add-event" /* ADD_TIMELINE_EVENT */, stringify(payload));
    });
  }
  bridge.on("user-app:connected" /* APP_CONNECTED */, () => {
    subscribeDevToolsApiUpdated();
  });
  if (isBrowser) {
    window.addEventListener("message", (e) => {
      const event = e.data;
      if (event.target === "vue-devtools" && event.event === "toggle-app-record")
        subscribeDevToolsApiUpdated();
    });
  }
}
var BridgeRpc2 = class {
};

// src/bridge/index.ts
function registerBridgeRpc3(target2, options) {
  if (target2 === "devtools")
    registerBridgeRpc(options);
  else
    registerBridgeRpc2(options.bridge);
}

// src/vue-plugin.ts
import { inject, ref } from "vue";
function initDevToolsState() {
  const connected = ref(false);
  const clientConnected = ref(false);
  const componentCount = ref(0);
  const vueVersion = ref("");
  const tabs = ref([]);
  const commands = ref([]);
  const vitePluginDetected = ref(false);
  const appRecords = ref([]);
  const activeAppRecordId = ref("");
  function init() {
    BridgeRpc.getDevToolsState().then(({ data }) => {
      connected.value = data.connected;
      clientConnected.value = data.clientConnected;
      vueVersion.value = data.vueVersion || "";
      tabs.value = data.tabs;
      commands.value = data.commands;
      vitePluginDetected.value = data.vitePluginDetected;
      appRecords.value = data.appRecords;
      activeAppRecordId.value = data.activeAppRecordId;
    });
    BridgeRpc.on.devtoolsStateUpdated((payload) => {
      connected.value = payload.connected;
      clientConnected.value = payload.clientConnected;
      vueVersion.value = payload.vueVersion || "";
      appRecords.value = payload.appRecords;
      activeAppRecordId.value = payload.activeAppRecordId;
    });
  }
  return {
    init,
    restore: init,
    vueVersion,
    connected,
    clientConnected,
    componentCount,
    tabs,
    commands,
    vitePluginDetected,
    appRecords,
    activeAppRecordId
  };
}
function initDevToolsBridge(_bridge) {
  const bridge = ref(_bridge);
  function restore(_bridge2) {
    bridge.value = _bridge2;
  }
  return {
    bridge,
    restore
  };
}
var VueDevToolsBridgeSymbol = Symbol("VueDevToolsBridgeSymbol");
var VueDevToolsStateSymbol = Symbol("VueDevToolsStateSymbol");
function createDevToolsVuePlugin(pluginOptions) {
  return {
    install(app, options) {
      const { bridge: _bridge, viewMode: _viewMode } = pluginOptions;
      const viewMode = ref(_viewMode);
      const state = initDevToolsState();
      const bridgeContext = initDevToolsBridge(_bridge);
      state.init();
      app.provide(VueDevToolsBridgeSymbol, bridgeContext.bridge);
      app.provide(VueDevToolsStateSymbol, state);
      app.provide("viewMode", viewMode);
      bridgeContext.bridge.value.on("toggle-view-mode", (v) => {
        viewMode.value = v;
      });
      app.config.globalProperties.__VUE_DEVTOOLS_UPDATE__ = (_bridge2) => {
        bridgeContext.restore(_bridge2);
        state.restore();
      };
    }
  };
}
function useDevToolsBridge() {
  return inject(VueDevToolsBridgeSymbol);
}
function useDevToolsBridgeRpc() {
  return BridgeRpc;
}
function useDevToolsState() {
  return inject(VueDevToolsStateSymbol);
}

// src/injection.ts
import { onDevToolsConnected } from "@vue/devtools-kit";

// src/handshake.ts
import { devtools as devtools2 } from "@vue/devtools-kit";
var HandShakeEvents = /* @__PURE__ */ ((HandShakeEvents2) => {
  HandShakeEvents2["SYN"] = "syn";
  HandShakeEvents2["SYN_ACK"] = "syn-ack";
  HandShakeEvents2["ACK"] = "ack";
  return HandShakeEvents2;
})(HandShakeEvents || {});
var HandShake = class {
  constructor(bridge) {
    this.socket = bridge;
  }
};
var HandShakeClient = class extends HandShake {
  constructor(bridge) {
    super(bridge);
  }
  onnConnect() {
    return new Promise((resolve) => {
      this.socket.emit("syn" /* SYN */);
      const timer = setInterval(() => {
        this.socket.emit("syn" /* SYN */);
      }, 300);
      this.socket.on("syn-ack" /* SYN_ACK */, () => {
        clearInterval(timer);
        this.socket.emit("ack" /* ACK */);
        devtools2.state.clientConnected = true;
        resolve();
      });
    });
  }
};
var HandShakeServer = class extends HandShake {
  constructor(bridge) {
    super(bridge);
  }
  onnConnect() {
    return new Promise((resolve) => {
      this.socket.on("syn" /* SYN */, () => {
        this.socket.emit("syn-ack" /* SYN_ACK */);
        const timer = setInterval(() => {
          this.socket.emit("syn-ack" /* SYN_ACK */);
        }, 300);
        this.socket.on("ack" /* ACK */, () => {
          clearInterval(timer);
          resolve();
        });
      });
    });
  }
};

// src/injection.ts
function prepareInjection(bridge) {
  registerBridgeRpc3("user-app", {
    bridge
  });
  new HandShakeClient(bridge).onnConnect().then(() => {
    bridge.on("devtools:client-ready" /* CLIENT_READY */, () => {
      onDevToolsConnected(() => {
        bridge.emit("user-app:connected" /* APP_CONNECTED */);
      });
    });
  });
}

// src/client.ts
import { isBrowser as isBrowser2, target } from "@vue/devtools-shared";
function setDevToolsClientUrl(url) {
  target.__VUE_DEVTOOLS_CLIENT_URL__ = url;
}
function getDevToolsClientUrl() {
  var _a;
  return (_a = target.__VUE_DEVTOOLS_CLIENT_URL__) != null ? _a : (() => {
    if (isBrowser2) {
      const devtoolsMeta = document.querySelector("meta[name=__VUE_DEVTOOLS_CLIENT_URL__]");
      if (devtoolsMeta)
        return devtoolsMeta.getAttribute("content");
    }
    return "";
  })();
}
export {
  Bridge,
  BridgeEvents,
  BridgeRpc as DevToolsRpc,
  HandShakeClient,
  HandShakeEvents,
  HandShakeServer,
  BridgeRpc2 as UserAppRpc,
  createDevToolsVuePlugin,
  getDevToolsClientUrl,
  prepareInjection,
  registerBridgeRpc3 as registerBridgeRpc,
  setDevToolsClientUrl,
  setupViteRPCClient,
  setupViteRPCServer,
  useDevToolsBridge,
  useDevToolsBridgeRpc,
  useDevToolsState
};
