// src/vite-rpc/assets.ts
import fsp from "node:fs/promises";
import fg from "fast-glob";
import { join, resolve } from "pathe";
import { imageMeta } from "image-meta";
import { debounce } from "perfect-debounce";
var defaultAllowedExtensions = [
  "png",
  "jpg",
  "jpeg",
  "gif",
  "svg",
  "webp",
  "ico",
  "mp4",
  "ogg",
  "mp3",
  "wav",
  "mov",
  "mkv",
  "mpg",
  "txt",
  "ttf",
  "woff",
  "woff2",
  "eot",
  "json",
  "js",
  "jsx",
  "ts",
  "tsx",
  "md",
  "mdx",
  "vue",
  "webm"
];
function guessType(path) {
  if (/\.(png|jpe?g|jxl|gif|svg|webp|avif|ico|bmp|tiff?)$/i.test(path))
    return "image";
  if (/\.(mp4|webm|ogv|mov|avi|flv|wmv|mpg|mpeg|mkv|3gp|3g2|ts|mts|m2ts|vob|ogm|ogx|rm|rmvb|asf|amv|divx|m4v|svi|viv|f4v|f4p|f4a|f4b)$/i.test(path))
    return "video";
  if (/\.(mp3|wav|ogg|flac|aac|wma|alac|ape|ac3|dts|tta|opus|amr|aiff|au|mid|midi|ra|rm|wv|weba|dss|spx|vox|tak|dsf|dff|dsd|cda)$/i.test(path))
    return "audio";
  if (/\.(woff2?|eot|ttf|otf|ttc|pfa|pfb|pfm|afm)/i.test(path))
    return "font";
  if (/\.(json[5c]?|te?xt|[mc]?[jt]sx?|md[cx]?|markdown|ya?ml|toml)/i.test(path))
    return "text";
  return "other";
}
function setupAssetsRPC(config) {
  const server = config.server;
  const getRpcServer = config.getRpcServer;
  const _imageMetaCache = /* @__PURE__ */ new Map();
  let cache = null;
  const extensions = defaultAllowedExtensions;
  async function scan() {
    const dir = resolve(config.root);
    const baseURL = config.base;
    const files = await fg([
      // image
      "**/*.(png|jpg|jpeg|gif|svg|webp|avif|ico|bmp|tiff)",
      // video
      "**/*.(mp4|webm|ogv|mov|avi|flv|wmv|mpg|mpeg|mkv|3gp|3g2|m2ts|vob|ogm|ogx|rm|rmvb|asf|amv|divx|m4v|svi|viv|f4v|f4p|f4a|f4b)",
      // audio
      "**/*.(mp3|wav|ogg|flac|aac|wma|alac|ape|ac3|dts|tta|opus|amr|aiff|au|mid|midi|ra|rm|wv|weba|dss|spx|vox|tak|dsf|dff|dsd|cda)",
      // font
      "**/*.(woff2?|eot|ttf|otf|ttc|pfa|pfb|pfm|afm)",
      // text
      "**/*.(json|json5|jsonc|txt|text|tsx|jsx|md|mdx|mdc|markdown|yaml|yml|toml)"
    ], {
      cwd: dir,
      onlyFiles: true,
      ignore: [
        "**/node_modules/**",
        "**/dist/**",
        "**/package-lock.*",
        "**/pnpm-lock.*",
        "**/pnpm-workspace.*"
      ]
    });
    cache = await Promise.all(files.map(async (path) => {
      const filePath = resolve(dir, path);
      const stat = await fsp.lstat(filePath);
      path = path.startsWith("public/") ? path.slice(7) : path;
      return {
        path,
        publicPath: join(baseURL, path),
        filePath,
        type: guessType(path),
        size: stat.size,
        mtime: stat.mtimeMs
      };
    }));
    return cache;
  }
  function watchAssets() {
    const debouncedAssetsUpdated = debounce(() => {
      getRpcServer().assetsUpdated();
    }, 100);
    server.watcher.on("all", (event) => {
      if (event !== "change")
        debouncedAssetsUpdated();
    });
  }
  watchAssets();
  return {
    async getStaticAssets() {
      return await scan();
    },
    async getImageMeta(filepath) {
      if (_imageMetaCache.has(filepath))
        return _imageMetaCache.get(filepath);
      try {
        const meta = imageMeta(await fsp.readFile(filepath));
        _imageMetaCache.set(filepath, meta);
        return meta;
      } catch (e) {
        _imageMetaCache.set(filepath, void 0);
        console.error(e);
        return void 0;
      }
    },
    async getTextAssetContent(filepath, limit = 300) {
      try {
        const content = await fsp.readFile(filepath, "utf-8");
        return content.slice(0, limit);
      } catch (e) {
        console.error(e);
        return void 0;
      }
    },
    assetsUpdated() {
    }
  };
}

// src/vite-rpc/graph.ts
import { debounce as debounce2 } from "perfect-debounce";
function setupGraphRPC(options) {
  const { rpc, server, getRpcServer } = options;
  const debouncedModuleUpdated = debounce2(() => {
    getRpcServer().moduleUpdated();
  }, 100);
  server.middlewares.use((req, res, next) => {
    debouncedModuleUpdated();
    next();
  });
  return {
    async getGraph() {
      const list = await rpc.list();
      const modules = (list == null ? void 0 : list.modules) || [];
      return modules;
    },
    moduleUpdated: () => {
    }
  };
}
export {
  setupAssetsRPC,
  setupGraphRPC
};
